#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "input.h"

void reliable_read(raw_input_func_t* input_func, void* userdata, uint8_t* buf, size_t len)
{
	while (len > 0) {
		ssize_t bytes_read = input_func(userdata, buf, len);
		assert(bytes_read <= (ssize_t)len);

		// TODO: We need better error handling here. setjmp()/longjmp()
		// should hopefully do the trick, but we need to take care for
		// suspension.
		if (bytes_read == (ssize_t)-1) {
			fprintf(stderr, "Input function returned error\n");
			exit(1);
		}
		if (bytes_read == 0) {
			fprintf(stderr, "Premature EOF\n");
			exit(1);
		}

		buf += bytes_read;
		len -= bytes_read;
	}
}

uint8_t read_uint8(raw_input_func_t* input_func, void* userdata)
{
	uint8_t ret;
	reliable_read(input_func, userdata, &ret, 1);
	return ret;
}

uint16_t read_uint16(raw_input_func_t* input_func, void* userdata)
{
	uint8_t buf[2];
	reliable_read(input_func, userdata, buf, 2);
	return (buf[0] << 8) | buf[1];
}

